"use strict";
let minCoins = [];
let secWs;
self.onmessage = function (e) {
    if (e.data === 'start') {
        secWs = startWebSocket();
    }
    if (e.data.indexOf("min") == 0) {
        let ar = e.data.split("/");
        let symbol = ar[1];
        let it = setInterval(() => {
            startMin(symbol);
        }, 1000 * 60);
        startMin(symbol);
        let ws = startOrderBook(symbol);
        minCoins.push({ symbol: ar[1], interval: it, ws: ws });
    }
    if (e.data.indexOf("minClose") == 0) {
        let ar = e.data.split("/");
        let symbol = ar[1];
        for (let n = minCoins.length - 1; n >= 0; n--) {
            if (minCoins[n].symbol == symbol) {
                minCoins[n].ws.close();
                clearInterval(minCoins[n].interval);
                console.log("Clear interval min : " + symbol);
                minCoins.splice(n, 1);
            }
        }
    }
    if (e.data == "clear") {
        for (let n = 0; n < minCoins.length; n++) {
            clearInterval(minCoins[n].interval);
            minCoins[n].ws.close();
        }
        minCoins = [];
        secWs.close();
    }
};
function f(num) {
    return num.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}
function ff(str) {
    return parseFloat(str.replace(/,/g, ''));
}
function crRange2Value(range1Start, range1End, range1Value, range2Start, range2End) {
    let ratio = (range1Value - range1Start) / (range1End - range1Start);
    return ((range2End - range2Start) * ratio) + range2Start;
}
class CCoinGraph {
    constructor() {
        this.min = Number.MAX_VALUE;
        this.max = Number.MIN_VALUE;
        this.history = [];
    }
    pushValue(price) {
        this.history.push(price);
        if (this.history.length > 300) {
            this.history.splice(0, 1);
        }
        if (this.min > price) {
            this.min = price;
        }
        if (this.max < price) {
            this.max = price;
        }
    }
    toData() {
        let data = { min: this.min, max: this.max, history: new Array() };
        for (let n = 0; n < this.history.length; n++) {
            data.history.push({ x: n, y: 200 - crRange2Value(this.min, this.max, this.history[n], 0, 200) });
        }
        return data;
    }
}
function startWebSocket() {
    const ws = new WebSocket('wss://stream.binance.com:9443/ws/!ticker@arr');
    let setSymbol = new Set();
    let mapGraph = new Map();
    let map = new Map();
    ws.onmessage = function (event) {
        const data = JSON.parse(event.data);
        for (let n = 0; n < data.length; n++) {
            map.set(data[n].s, {
                symbol: data[n].s,
                lastPrice: f(parseFloat(data[n].c)),
                prevClosePrice: f(parseFloat(data[n].x)),
                priceChange: f(parseFloat(data[n].p)),
                priceChangePercent: f(parseFloat(data[n].P)),
                bidPrice: f(parseFloat(data[n].b)),
                bidQty: f(parseFloat(data[n].B)),
                askPrice: f(parseFloat(data[n].a)),
                askQty: f(parseFloat(data[n].A)),
                openPrice: f(parseFloat(data[n].o)),
                highPrice: f(parseFloat(data[n].h)),
                lowPrice: f(parseFloat(data[n].l)),
                volume: f(parseFloat(data[n].v)),
                quoteVolume: f(parseFloat(data[n].q))
            });
        }
        let arr = [];
        if (setSymbol.size == 0) {
            for (let n = 0; n < data.length; n++) {
                if (data[n].c >= 1) {
                    setSymbol.add(data[n].s);
                }
            }
            setSymbol.forEach((v) => {
                let info = map.get(v);
                if (info) {
                    let cg = new CCoinGraph();
                    cg.pushValue(ff(info.lastPrice));
                    mapGraph.set(info.symbol, cg);
                }
            });
        }
        else {
            setSymbol.forEach((v) => {
                let info = map.get(v);
                if (info) {
                    let cg = mapGraph.get(v);
                    if (cg) {
                        cg.pushValue(ff(info.lastPrice));
                        info.graph = cg.toData();
                    }
                    arr.push(info);
                }
            });
            self.postMessage({ kind: "sec", data: arr });
        }
    };
    ws.onerror = function (error) {
        console.error('WebSocket error:', error);
        self.postMessage({ error: 'WebSocket connection error' });
    };
    ws.onclose = function () {
        console.log('Sec WebSocket closed');
    };
    return ws;
}
async function startMin(symbol) {
    let response = await fetch(`https://api.binance.com/api/v3/klines?symbol=${symbol}&interval=1m&limit=1440`);
    const data = await response.json();
    const lastTimestamp = data[0][0];
    response = await fetch(`https://api.binance.com/api/v3/klines?symbol=${symbol}&interval=1m&endTime=${lastTimestamp}&limit=440`);
    const data2 = await response.json();
    let arr = [];
    for (let n = 0; n < data2.length; n++) {
        arr.push(data2[n]);
    }
    for (let n = 0; n < data.length; n++) {
        arr.push(data[n]);
    }
    self.postMessage({ kind: "min", symbol: symbol, data: arr });
}
function startOrderBook(symbol) {
    const wsUrl = `wss://stream.binance.com:9443/ws/${symbol.toLowerCase()}@depth`;
    const ws = new WebSocket(wsUrl);
    ws.onmessage = (event) => {
        const orderBook = JSON.parse(event.data);
        let o = { bid: [], ask: [] };
        for (let n = 0; n < orderBook.b.length; n++) {
            o.bid.push(f(parseFloat(orderBook.b[n][0])) + "(" + f(parseFloat(orderBook.b[n][1])) + ")");
        }
        for (let n = 0; n < orderBook.a.length; n++) {
            o.ask.push(f(parseFloat(orderBook.a[n][0])) + "(" + f(parseFloat(orderBook.a[n][1])) + ")");
        }
        self.postMessage({ kind: "order", symbol: symbol, data: o });
    };
    ws.onerror = (error) => {
        console.error('WebSocket Error:', error);
    };
    ws.onclose = () => {
        console.log('Orderbook WebSocket closed : ' + symbol);
    };
    return ws;
}
